lexer grammar LinkFetchExpLexer;

// lexer grammar tokens
// mono syllabic
COMMA : ',' ;
SEMICOLON : ';' ;
COLON : ':' ;
DASH : '-' ;
O_BRACK : '[' ;
C_BRACK : ']' ;
O_CURLBRACK : '{' ;
C_CURLBRACK : '}' ;
ARROW : '->' ;
LT : '<'                        -> pushMode(HTMLTAG_MODE) ;
// GT : '>' ; // defined in mode html since it is only used there
EQUAL : '=' ;
SLASH : '/' ;

START_JSON_LINKS_MODE : 'at' WS? '('  -> pushMode(JSON_LINKS_MODE) ;
START_CONN_PROP : ('@props' | '@connProps' | '@connectionProps' | '@connectionProperties') 
                    WS? '[' -> pushMode(CONN_PROPERTIES_MODE) ; //old inflexible val '@props['
DIRECTIVE_START : '@directive' WS? '[' ;
ID_TAG : '#id:' | '@id:' ;
SITE_TAG : '@site:' ;
PARAMS_TAG : ('@params:' | '@parameters:')    -> pushMode(DIRECTIVE_PARAMS_MODE) ;
ON_TAG : 'on' WS? '('       -> pushMode(ON_OPTIONS_MODE) ;
LABEL_TAG : ('@label:' | '@label')    -> pushMode(LABEL_MODE) ;
WEBSITE : ('ftp' | 'http' | 'https') '://' [a-zA-Z0-9] ~(';' | ',' | ' ')* '.' ~(';' | ',' | ' ')+ ;
URLSCHEME_SLASH : '://' ;
URLSCHEME : '$scheme' | '$protocol' ;
HOST_VAR : '$host' ;
PORT_VAR : '$port' ;
PATH_VAR : '$path' ;
QUERY_VAR : '$query' ;
URLREGEX_GROUP_VAR : '$' [0-9]+ ;
OPEN : 'open' ;
PAGE : 'page' | 'document' | 'doc' | 'content' ;
GET : 'GET' ;
POST : 'POST' ;
LABEL_ID : 'labelid' ;
TO : 'to' ;
IN : 'in' ;
AS : 'as' ;
ABS : 'abs' ;
OR : 'or' ;
AND : 'and' ;
ELSE : 'else' ;
ALSO : 'also' ;
ANY : 'any' ;
ONLY : 'only' ;
THOSE : 'those' ;
NEXT : 'next' ;
GOTO : 'goto' ;
WAIT : 'wait' ;
EACH : 'each' ;
FETCH : 'fetch' ;
RETURN : 'return' ;
ALL : 'all' ;
FIRST : 'first' ;
LAST : 'last' ;
FROM : 'from' ;
YIELDING : 'yielding' ;
WITH : 'with' ;
SUBMIT : 'submit' ;
HTMLFORM : 'htmlform' ;
MAKE : 'make' ;
API : 'api' ;
CALL : 'call' ;
JSVAR : '$jsvar:' ;
HTMLVAL : '$htmlval:' ;
VARIABLE : '${' [a-zA-Z0-9_.-]+ '}' ;
//FORM_VARIABLE : '$' (('\\,' |  ~[, ":;{/])+ | ('"' ('\\"' | ~'"')+ '"') ) ; // old & complicated, don't know why
FORM_VARIABLE : '$' ([a-zA-Z0-9_.-]+ | ('"' ('\\"' | ~'"')+ '"') ) ; // matches for e.g. $id or $"id"
PARAMETERS : 'parameters' ;
POST_ENCODING : 'postEncoding' | 'postencoding' ;
CUSTOM_HANDLER : 'customhandler' | 'customHandler';
VIDEO_LINKS : 'videolinks' ;
TEXTLINKS : 'textlinks' ;
COMMENTLINKS : 'commentlinks' ;
CDATALINKS : 'cdatalinks' ;
JSOBFUSCATEDLINKS : 'jsobfuscatedlinks' ;
ENCODEDLINKS : 'encodedlinks' ;
JSONLINKS : 'jsonlinks' ;
LINKS : 'links' ;
PATTERN : 'pattern' ;
USING : 'using' ;
CONN_PROPS_STR : 'connectionPropsID' | 'connectionPropsId' ;
DIRECT : 'directly' ;
INDIRECT : 'indirectly' ;
PROGRAMMATICALLY : 'programmatically' ;
ABSOLUTE : 'absolute' ;
REMOVE : 'remove' ;
MATCHING : 'matching' ;
CUSTOMNAME : 'customname' ;
CUSTOMSUBFOLDER : 'customsubfolder' | 'customfolder' ;
VALUE_LITERAL : 'value' ;
FILE : 'file' ;
EXTENSION : 'extension' ;
WORKOUT : 'workout' ;
NONE : 'none' ;
NULL : 'null' ;
URL_VERB : 'url' ;
REWRITE : 'rewrite' ;
CONTENT_TYPE : 'contenttype' | 'content-type' | 'contentType' ;
PAGE_TITLE_LIT : 'pagetitle' ;
NEXTPAGE: 'nextpage' ;
LOCATION: 'location' ;
VIA: 'via' ;
JAVASCRIPT: 'javascript' ;
FOLLOW_NEXTPAGES_OF_NEXTPAGES : 'followNextPagesOfNextPages' ;
KEEP_LINKS_REFERRAL_HISTORY : 'keepLinksReferralHistory' ;
CLOUDFLARE_PROTECTED : 'cloudflareProtected' ;
ELEVATE_INNER_GRAMMARS : 'elevateInnerGrammars' ;
SECS : 'sec' 's'? ;
SECONDS : 'second' 's'? ;
MINUTES : 'minute' 's'? ;
HOURS : 'hour' 's'? ;
NUMBER : [0-9]+ ;
WORD : [a-zA-Z0-9-]+ ;
TAG_LINK_SOURCE : WORD '_' WORD ;
SELF_ATTRIBUTE_LINK : '.attribute_' WORD ;
SELF_TEXT_VAL : '.textval' ;
URLSTRING : '"' 'http' 's'? '://' [a-zA-Z0-9]+ ('\\"' | ~'"')* '"' ;
QUOTED_STR : '"' ('\\"' | ~'"')* '"' ;       //'"' [ a-zA-Z0-9._\\_-;@':]* '"' ;
SINGLE_QUOTED_STR : '\'' ('\\\'' | ~'\'')* '\'' ;
REGEXVAL : ('~' ('\\~' | ~'~')+ '~') 
         | ('#' ([ a-zA-Z0-9._,|/\\?*+^$"':;<>!=()&%{}] | '-' | '[' | ']' | '\\#')+ '#'); 
// example of regex vals: ~\.com~ and #https?://\w+#

WS : [ \t\r\n]+ -> skip ;
COMMENT :   '/*' .*? '*/' -> skip ;


// mode for html tag
mode HTMLTAG_MODE;

H_WORD : WORD ;
H_ASSIGN : EQUAL ;
H_HTML_ATTR : '$html' ;
H_TEXT_ATTR : '$text' ;
ENDTAG : '>'               -> popMode ; // ie '>' to end html tag
ATTRIBVAL : QUOTED_STR | SINGLE_QUOTED_STR;
REGEX_ATTRIBVAL : REGEXVAL ;

H_WS : WS       -> skip;
H_COMMENT : COMMENT -> skip ;


// mode declaring directive params
mode DIRECTIVE_PARAMS_MODE;

D_WS : WS       -> skip;
D_COMMENT : COMMENT -> skip ;

O_CBR_P : O_CURLBRACK ;
PARAMETER : [a-zA-Z0-9] [a-zA-Z0-9-_]+ ;
P_COMMA : COMMA ;
P_OR : '|' ;
O_PAREN_P : '(' ;
C_PAREN_P : ')' ;
D_QUOTED_STR : QUOTED_STR | SINGLE_QUOTED_STR ;
C_CBR_P : C_CURLBRACK   -> popMode ; //end mode


// on options mode
mode ON_OPTIONS_MODE;

O_WS : WS       -> skip;
O_COMMENT : COMMENT -> skip ;

O_PARAMETER : PARAMETER ;
O_COMMA : COMMA ;
O_COLON : COLON ;
O_EQ_SYM : EQUAL ;
O_QUOTED_STR : QUOTED_STR | SINGLE_QUOTED_STR ;
C_PAREN_O : ')'     -> popMode ; //end mode

// mode for label ie code labeling
mode LABEL_MODE;

L_WS : WS -> skip ;
L_COMMENT : COMMENT -> skip ;

LABEL : 'label' ;
OP_BRACK : O_BRACK ;
ID : 'id' ;
EQ_SYM : EQUAL ;
L_COLON : COLON ;
IDVAL : NUMBER ;
SEP : COMMA ;
END_LABEL : C_BRACK             -> popMode ;
LABELVAL : QUOTED_STR | SINGLE_QUOTED_STR ; //'"' ('\\"' | ~('"' | '[' | ']'))* '"' ;
URL_REGEX : 'urlregex' ;
URL_REGEXVAL : REGEXVAL ;


// mode for json links
mode JSON_LINKS_MODE;
J_WS : WS -> skip ;
J_COMMENT : COMMENT -> skip ;

J_SEP : COMMA ;
J_COLON : COLON ;
J_DASH : DASH ;
J_O_BRACK : O_BRACK;
J_C_BRACK : C_BRACK;
J_ARROW : ARROW ;
J_ALL : ALL;
J_ABS : ABS ;
J_ABSOLUTE : ABSOLUTE ;
J_OBJECT : '$data' | '$jsonData' | '$json_data';
J_NUMBER : NUMBER ; // must be before j_ident to enable correct matching
J_QUOTED_STR : QUOTED_STR | SINGLE_QUOTED_STR ;
J_REGEXVAL : REGEXVAL ;
J_IDENT : [a-zA-Z0-9_]+ ;
END_JSON_LINKS_MODE : ')'              -> popMode ;



// mode for conneciton properties
mode CONN_PROPERTIES_MODE;

P_WS : WS -> skip ;
P_COMMENT : COMMENT -> skip ;

QUOTE : '"' ;
ASSIGN : EQUAL ;
O_PAREN : '(' ;
C_PAREN : ')' ;
O_CBR : O_CURLBRACK ;
C_CBR : C_CURLBRACK ;
P_VARIABLE : VARIABLE ;   //'{' (TEXT_CHAR+ | NUM) '}' ;
P_QUOTED_STR : QUOTED_STR | SINGLE_QUOTED_STR ;
END_CONN_PROP : C_BRACK              -> popMode ;

NAMESPACE : '@namespace:' ;
ID_START : '#id:' | '@id:' ;
HEADER_FIELD : '@headerField:' | '@hf:' ;
POST_PARAMETER : '@postParameter:' | '@postParam:' | '@pp:' ;
HOTLINK_AWARE : '@hotlinkAware' ;
REQUEST_METHOD : '@requestMethod:' ;
AUTHORIZATION_REQUIRED : '@authorizationRequired' ;
JAVASCRIPT_LOADING_REQUIRED : '@javascriptLoadingRequired' ;

NUM : [0-9]+ ;
REQ_WORD : 'GET' | 'POST' | [A-Z]+ ;

fragment TEXT_CHAR 
    : '\\"' | '\\#' | '\\[' | '\\]' | '\\{' | '\\}'
    | ~('"' | '@' | '#' | '{' | '}') 
    ;

